#include "PbarSI.h"
#include "TMath.h"
#include <iostream>
#include <cmath>
#if 1
#include "TFile.h"
#include "TROOT.h"
#endif

ClassImp(TPC::PbarSI)
namespace TPC {
// C-tor
PbarSI::PbarSI(double sigma,  // sigma_d    [mm] 
               double l    ,  // pad length [mm]
               double dy   )  // Delta Y    [mm]
      : fEta(sigma,l),
        fDy(dy),
	fHPbarSI("pbarsi","",5000,0.,5000.)
{
#if 0
    static const double Ar[] = { 65.60, 15.00, 6.40, 3.50, 2.25,
                                  1.55,  1.05, 0.81, 0.61, 0.49,
                                  0.39,  0.30, 0.25, 0.20, 0.16,
                                  0.12,  0.095, 0.075, 0.063, 21.6 };
#else
#endif
    const int kMax = 3000;
    double sum = 0;
    fPSI.reserve(kMax);
    for (int i=0; i<kMax; i++) {
      double val;
#if 0
      if (i<19) val = Ar[i];
      else      val = Ar[19]/(i+1)/(i+1);
#else
      static TH1D *hp = nullptr;
      if (!hp) {
         auto ncfilep = new TFile("Data/nc.electron.5GeV.root");
	 std::cerr << "Cluster size data from " << ncfilep->GetName() << std::endl;
         hp = static_cast<TH1D *>(gROOT->FindObject("hnc"));
      }
#if 0
      val = hp->GetBinContent(i);
#else
      val = hp->GetBinContent(i+1);
#endif
#endif
      sum += val;
      fPSI.push_back(val);
    }
    for (int i=0; i<kMax; i++) fPSI[i] /= sum; // normalize to unity

    const int kMax2 = 1000;
    fPbarSI.reserve(kMax2);
    std::cerr << "+--------+---------+---------+---------+---------+\r";
    for (int k=0; k<kMax2; k++) {
        static const double kNsig = 5; // +/-kNsig sigmas from row boundaries
        double ymin = TMath::Max(-l/2-kNsig*sigma,-fDy/2);
        double ymax = 0.;
	if (!k) {
            ymin = -fDy/2;
            ymax = TMath::Min(-l/2+kNsig*sigma,0.);
	}
        TF1 pbsi("pbsi",this,ymin,ymax,1);
	static const double eps = 1.e-5;
#if 0
	double kd = k;
	double sum = pbsi.Integral(ymin,ymax,&kd,eps);
#else
	pbsi.SetParameter(0,k);
	double sum = pbsi.Integral(ymin,ymax,eps);
#endif
	sum *= 2/fDy;
        fPbarSI.push_back(sum);
	fHPbarSI.SetBinContent(k+1,sum);
	if (!(k%20)) std::cerr << "=";
    }
    std::cerr << "=" << std::endl 
              << "PbarSI::PbarSI() .... Initialization finished!"      << std::endl
              << "===================================================" << std::endl;
}

// public member functions
double PbarSI::GetSigma() const
{
    return fEta.GetParameter(0);
}

double PbarSI::GetL() const
{
    return fEta.GetParameter(1);
}

// public member functions
double PbarSI::operator()(unsigned int k, double y) const
{
    double eta = fEta(y, 0);
    double pbs = 0;
    for (unsigned int m=(!k?1:k); m<fPSI.size(); m++) {
        pbs += GetPSI(m)*(TMath::BinomialI(eta,m,k)-TMath::BinomialI(eta,m,k+1));
    }
    pbs /= operator()(k);
    return pbs;
}

double PbarSI::operator()(unsigned int k) const
{
    return k < fPbarSI.size() ? fPbarSI[k] : 0;
}

double PbarSI::PbarY(double *y, double *k) const
{
    int ki = k[0];
    return operator()(ki,*y);
}

double PbarSI::GetRandom() const
{
    return fHPbarSI.GetRandom();
}

// private member functions
double PbarSI::GetPSI(unsigned int m) const
{
    return m ? (m < fPSI.size() ? fPSI[m-1] : 0) : 0;
}

double PbarSI::operator()(double *y, double *par) const
{
    int    k   = par[0];
    double eta = fEta(*y, 0);
    double pbs = 0;
    for (unsigned int m=(!k?1:k); m<fPSI.size(); m++) {
        pbs += GetPSI(m)*(TMath::BinomialI(eta,m,k)-TMath::BinomialI(eta,m,k+1));
    }
    return pbs;
}
}
