#ifndef PBARSI_H
#define PBARSI_H
#include "Prf.h"

#include <TH1D.h>
#include <TObject.h>

namespace TPC {
class PbarSI : public TObject {
public:
    PbarSI(double sigma = 0.27,  // sigma_d    [mm] 
           double l     = 5.20,  // pad length [mm]
           double dy    = 20.0); // Delta Y    [mm]

    double GetSigma() const;
    double GetL    () const;
    double GetDy   () const { return fDy; }

    double operator()(unsigned int k, double y) const;
    double operator()(unsigned int k)           const;
    double operator()(double *y, double *k)     const;
    double PbarY     (double *y, double *k)     const;

    double GetRandom () const;
    TH1D  &GetHPbarSI() { return fHPbarSI; }

private:
    double GetPSI(unsigned int m) const;

private:
    Prf                 fEta;     // eta(y)
    double              fDy;      // Delta Y
    TH1D                fHPbarSI; // pbar_SI[k] histogram
    std::vector<double> fPSI;     // P_SI[m]
    std::vector<double> fPbarSI;  // Pbar_SI[k]

    ClassDef(PbarSI, 1)   // Secondary Ionization
};
}
#endif
