// Usage:
// $ root -l
// ....
// [0] .L PlotPhiDep.C
// [1] PlotPhiDep("sg.dat")->Draw("apc2);
//
//
//
class Data {
public:
  Data(double phi = 0., double sigd = 0., double sigx = 0.)
       : fPhi(phi), fSigd(sigd), fSigx(sigx) 
  {
    //cerr << "phi = " << fPhi << " sigd = " << fSigd << " sigx = " << fSigx << endl;
  }

  inline double GetPhi () const { return fPhi;  }
  inline double GetSigd() const { return fSigd; }
  inline double GetSigx() const { return fSigx; }


  void Print() const
  {
    cerr << setw(6) << setprecision(3) << setiosflags(ios::fixed) << fPhi  << " "
         << setw(5) << setprecision(2) << setiosflags(ios::fixed) << fSigd << " "
         << setw(8) << setprecision(5) << setiosflags(ios::fixed) << fSigx
         << endl;
  }

private:
  double fPhi;  // phi     [rad]
  double fSigd; // sigma_d [mm]
  double fSigx; // sigma_x [mm]
};

bool operator<(const Data &a, const Data &b)
{
  return a.GetPhi() < b.GetPhi();
}

double Fun(const double *xv, const double *pv)
{
#if 0
  double sg2 = pv[0]*pv[0]
	     + (pv[2]/(12.*pv[1]/cos(xv[0]))) *tan(xv[0])*tan(xv[0]);
#else
  double sg2 = pv[0] 
	     + pv[3]/(pv[4]/cos(xv[0])) 
	     + (pv[2]/(12.*pv[1]/cos(xv[0]))) *tan(xv[0])*tan(xv[0]);
#endif
  return sqrt(sg2);
}

TMultiGraph *PlotPhiDep(const char *fname = "sg.dat")
{
  stringstream slscmd;
  slscmd << "sort -k 2 " << fname << ends; 
  TString list = gSystem->GetFromPipe(slscmd.str().data());

  const int kNpt = 100;
  vector<Data> data[kNpt];

  double phi, sigd, sigx;
  double sigds = -99999.;
  int    npt   = -1;
  stringstream ins(list.Data());
  // data format:  phi sig_d sig_x 
  while (ins >> phi >> sigd >> sigx) {
    if (sigd != sigds) { // new drift distance
      sigds = sigd; 
      npt++;
    }
    data[npt].push_back(Data(phi, sigd, sigx));
    if (phi != 0.) {
      data[npt].push_back(Data(-phi, sigd, sigx));
    }
  }
  npt++;

  auto mgp = new TMultiGraph;
  for (int ipt=0; ipt<npt; ipt++) {
    sort(data[ipt].begin(), data[ipt].end()); // sort by phi
    auto nphi = data[ipt].size();
    double xdata[nphi], ydata[nphi];
    int iphi = 0;
    for (const Data &dat : data[ipt]) {
      xdata[iphi] = dat.GetPhi ();
      ydata[iphi] = dat.GetSigx();
#if 0
      dat.Print();
#endif
      iphi++;
    }
    auto gp = new TGraph(iphi,xdata,ydata);
#if 0
    if (!ipt) mgp->Add(gp, "acp");
    else      mgp->Add(gp, "cp");
#else
    mgp->Add(gp);
#endif

    /////FIXME////////////////
    double L  = 5.36;  // [mm]
    double cd = 0.096; // [mm]
    //////////////////////////
    auto funp = new TF1("fun",&Fun,-5.,5.,5);
    funp->SetParameter(0,0.01); // [A] term
    funp->SetParameter(1,5.);  // Nheff
    funp->FixParameter(2,L*L);
    funp->FixParameter(3,cd*cd);
    funp->FixParameter(4,25); // Neff
    //funp->SetParameter(4,25.*25.); // Neff
    //gp->Fit(funp,"Q","",-5,5);
    gp->Fit(funp,"","",-5.,5.);
#if 1
    sigd = data[ipt].at(0).GetSigd();
    double dlen = pow(sigd/cd,2);
    cerr << "sigd = " << sigd << " dlen = " << dlen << endl
         << " Neff (F) = " << funp->GetParameter(4) 
         << " +/- "        << funp->GetParError (4)
         << " Nheff(F) = " << funp->GetParameter(1) 
         << " +/- "        << funp->GetParError (1)
         << endl;
#endif
  }
  return mgp;
}



