#include <TF1.h>
#include <TMath.h>
#include <TSystem.h>

struct MyFunc { 
    MyFunc(TGraph * gr): fGraph(gr) {}
    double operator() (double *x, double * )  const { 
       //double a = fFunc->GetXmin();
       return fGraph->Eval(*x);
    }
    TGraph* fGraph; 
};

TGraph *PlotTerm(double &sgx, const char *term = "bt", const char *cond = "terms.*tanf*.sigd*")
{
    static int  ncall = 0;
    ncall++;
    //--
    // List and sort tup files in the ascending order of xt
    //--
    stringstream slscmd;
    slscmd << "ls " << cond << "*.root | sort" << ends; 
#if 0
    cerr << "slscmd = " << slscmd.str() << endl;
#endif
    TString list = gSystem->GetFromPipe(slscmd.str().data());
#if 0
    cerr << list << endl;
#endif
    //--
    // Loop over tup files for different "xt" and save the "term" value for each "xt"
    //--
    stringstream slist(list.Data());
    string infile; // input filename
    double xt[10]; 
    double y [10];
    int n = 0;
    while (slist >> infile) {
        //--
        // Open input root file
        //--
        auto fp = new TFile(infile.data());
        fp->cd();
        auto evt = static_cast<TNtupleD *>(gROOT->FindObject("evt"));
        //--
        // Draw the requested "term"
        //--
        stringstream hst;
        hst << "h" << n << ends;
        stringstream cmdst;
        cmdst << term << ">>" << hst.str().data() << ends;
        evt->Draw(cmdst.str().data(),"k>0","goff");
        auto hp = static_cast<TH1D *>(gROOT->FindObject(hst.str().data()));
        y[n] = hp->GetMean();
        //--
        // Draw "xt" and get the xt value for this file
        //--
        hst.str("");
        hst.clear();
        hst << "hxt" << n << ends;
        cmdst.str("");
        cmdst.clear();
        cmdst << "xt>>" << hst.str().data() << ends;
        evt->Draw(cmdst.str().data(),"k>0","goff");
        auto hxtp = static_cast<TH1D *>(gROOT->FindObject(hst.str().data()));
        xt[n] = hxtp->GetMean();
#if 0
        cerr << " xt[" << n << "] = " << xt[n]
             << " y[" << n << "] = " << y[n] << endl;
#endif
        n++;
    }
    for (int i=1; i< n; i++) xt[i] /= 2*xt[n-1]; // normalize xt by pad pitch

    //--
    // Average the requested "term" over a single pad
    //--
    const int kNx = 32;
    double xdat[kNx+1], ydat[kNx+1];
    double xmin = -2.0;
    double xmax = +2.0;
    double dx   = (xmax-xmin)/kNx;
    for (int i=0; i<=kNx; i++) {
        double x   = xmin + i*dx;
	int    reg = x;
        double res = TMath::Abs(x - (int)x);
        if (res == 0.)         { ydat[i] = y[0]; xdat[i] = (x >= 0 ? reg+xt[0] : reg-xt[0]); }
        if (res == 0.125)      { ydat[i] = y[1]; xdat[i] = (x >= 0 ? reg+xt[1] : reg-xt[1]); }
        else if (res == 0.250) { ydat[i] = y[2]; xdat[i] = (x >= 0 ? reg+xt[2] : reg-xt[2]); }
        else if (res == 0.375) { ydat[i] = y[3]; xdat[i] = (x >= 0 ? reg+xt[3] : reg-xt[3]); }
        else if (res == 0.500) { ydat[i] = y[4]; xdat[i] = (x >= 0 ? reg+xt[4] : reg-xt[4]); }
        else if (res == 0.625) { ydat[i] = y[3]; xdat[i] = (x >= 0 ? reg+1-xt[3] : reg-1+xt[3]); }
        else if (res == 0.750) { ydat[i] = y[2]; xdat[i] = (x >= 0 ? reg+1-xt[2] : reg-1+xt[2]); }
        else if (res == 0.875) { ydat[i] = y[1]; xdat[i] = (x >= 0 ? reg+1-xt[1] : reg-1+xt[1]); }
#if 0
	cerr << x << " " << xdat[i] << " " << ydat[i] << endl;
#endif
    }
    //--
    // Plot the term as a function of xt over 4 pads
    //--
    TGraph *gr = new TGraph(kNx+1,xdat,ydat);
    gr->SetTitle("");

#if 0
    double ymax = 2.5*(y[0]+y[1]+y[2]+y[3]+y[4])/5;
#else
    double ymax = -99999.;
    for (int i=0; i<5; i++) {
        if (y[i]>ymax) ymax = y[0];
    }
    ymax *= 1.4;
#endif
    static TH1F *frame = nullptr;
    if (!frame) {
        frame = new TH1F("frame","",200,xmin/2,xmax/2);
        frame->SetMaximum(ymax);
        frame->SetStats(0);
        frame->SetMinimum(0.);
        frame->GetXaxis()->SetTitle("#tilde{x}/w");
        frame->GetXaxis()->SetTitleSize(0.045);
        frame->GetXaxis()->SetTitleOffset(0.7);
        frame->GetXaxis()->SetTitleFont(42);
        frame->GetYaxis()->SetTitle("#sigma_{x}^{2}(#tilde{x}) [#mum^{2}]");
        frame->GetYaxis()->SetTitleSize(0.045);
        frame->GetYaxis()->SetTitleOffset(1.1);
        frame->GetYaxis()->SetTitleFont(42);
    }
    gr->SetHistogram(frame);
    //--
    // Average the "term" over a single pad
    //--
    MyFunc *func  = new MyFunc(gr);
    stringstream ssfun;
    ssfun << "f2" << ncall << ends;
    TF1    *f2    = new TF1(ssfun.str().data(), func, xmin, xmax, 0, "MyFunc"); 
    static const double kEps = 1.e-7;
    double integ  = f2->Integral(-0.5,+0.5,kEps);
    sgx = sqrt(integ);
    cerr << " integ = " << integ << " [mm^2] "
         << " sgx = "   << sgx   << " [mm]" << endl;
    //--
    // Now return the "term" vs xt graph and exit
    //--
    return gr;
}
